import { WrapCpu } from "cpuemulator";
import $ from "jquery";

// creating the cpu object. Yeah.
let cpu = new WrapCpu();

// setting up the "interactive" editor
let textarea = $("#editing");
let highlights = $("#highlighting");

textarea.on({
  input: handleInput,
  keyup: handleInput,
  onchange: handleInput,
  scroll: handleScroll,
});

handleInput(); // immediately calling this the first time to highlight the sample code.

function handleInput() {
  var text = textarea.val();
  if (text != undefined) {
    var highlightedText = applyHighlights(text.toString());
    highlights.html(highlightedText);
  }
}

function applyHighlights(text: string) {
  let res = text
    .replace(
      /^(READ\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(WRITE\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(LOAD\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(LOAD\s+=\s*[-+]?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(STORE\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(ADD\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(ADD\s=\s*[-+]?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(SUB\s+\*?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(SUB\s=\s*[-+]?\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(JUMP\s+\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(JZERO\s+\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(
      /^(JGTZ\s+\d+)(\s*(#.*)?)?$/gm,
      '<mark class="goodcode">$1</mark>$2'
    )
    .replace(/^(HALT)(\s*(#.*)?)?$/gm, '<mark class="goodcode">$1</mark>$2')
    .replace(/(#.*\n?\$?)/gm, '<mark class="comment">$1</mark>');
  return res + "\n";
}

function handleScroll() {
  var scrollTop = textarea.scrollTop();
  var scrollLeft = textarea.scrollLeft();
  if (null != scrollTop && null != scrollLeft) {
    highlights.scrollTop(scrollTop);
    highlights.scrollLeft(scrollLeft);
  }
}

// setting up input box:

$("#inputqueue").on({
  input: restrictNumeric,
  paste: restrictNumeric,
});

function restrictNumeric() {
  var text = $("#inputqueue").val();
  if (typeof text == "string") {
    let newtext = text.replaceAll(/[^0-9 -]/g, "");
    $("#inputqueue").val(newtext);
  }
}

// setting up buttons.

$("#load").on({
  click: function () {
    $("#messagebox").html("Program loaded!");
    handleInput(); // immediately calling this the first time to highlight the sample code.
    cpu.reset();
    loadPrg();
    loadInput();
    writeMem();
    writeProgram();
    writeInput();
    writeOutput();
    highlightCurrent();
  },
});

$("#run").on({
  click: function () {
    $("#messagebox").html("Running the whole program!");

    loadInput(); // did the user add or change input?

    cpu.run();

    writeMem();
    writeOutput();
    writeInput();
    highlightCurrent();
  },
});

$("#run1").on({
  click: function () {
    $("#messagebox").html("Running a single step!");


    highlightCurrent();
    loadInput(); // did the user add or change input?
    cpu.one_step();
    writeMem();
    writeOutput();
    writeInput();
  },
});

$("#reset").on({
  click: function () {
    $("#messagebox").html("Resetting the whole machine!");
    cpu.reset();
    writeMem();
    writeInput();
    writeOutput();
    writeProgram();
    highlightCurrent();
  },
});

function writeMem() {
  let mem = cpu.showmem();

  let memtable = $("td");

  for (let index = 0; index < mem.length; index++) {
    memtable[index].innerHTML = mem[index].toString();
  }
}

function writeOutput() {
  let out = cpu.showoutput().join(" ");

  let outputbox = $("#outputqueue");
  outputbox.html(out);
}

function writeInput() {
  let input = cpu.showinput().reverse().join(" ");


  let inputbox = $("#inputqueue");

  inputbox.val(input);
}

function writeProgram() {
  let prog = cpu.showprogram().join("<br>");

  let progbox = $("#progbox");
  progbox.html(prog);

  // for (let index = 0; index < prog.length; index++) {
  //   progbox.val(progbox.val() + " " + prog[index]);
  // }
}

function loadInput() {
  // let input = cpu.showinput();

  let inputboxtext = $("#inputqueue").val();
  if (typeof inputboxtext == "string" && inputboxtext != "") {

    let inputqueue = inputboxtext.split(" ").reverse().map(Number);

    console.log(inputqueue);

    let arr = new Int32Array(inputqueue);

    cpu.set_input(arr);

    // for (let index = 0; index < inputqueue.length; index++) {
    //   if (inputqueue[index] != "") {
    //     cpu.add_input(Number(inputqueue[index].trim()));
    //   }
    
  }
}

function loadPrg() {
  let prg = textarea.val();

  if (typeof prg == "string") {
    cpu.load_program(prg);
  }
}

function highlightCurrent() {
  let current = cpu.showmem()[9];
  writeProgram();
  let progbox = $("#progbox");
  let text = progbox.html();

  let lines = text.toString().split("<br>");
  if (progbox != null && text != null && current >= 0 && current < lines.length) {
    lines[current] =
      '<mark class="goodcode">' + lines[current] + "</mark>";
    var newtext = lines.join("<br>");
    progbox.html(newtext);
  }
}
